function motorf(S,p,varargin)   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Skripta za crtanje sinhrone masine sa stalnim magnetima u FEMM42.
%   Moguce je proizvoljno menjati broj pari polova p i broj zlebova S.
%   Crta se minimalni deo masine potreban za validnu simulaciju, koji se
%   automatski odredjuje na osnovu unetih p i S. Linearno se skaliraju
%   dimenzije referentnog dizajna (s18p6). Pretpostavljena je radijalna
%   magnetizacija, a materijali su uzeti iz integrisane biblioteke.
%
%   Filip Cvejic
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (S>48)
    error('max(S)=48');
end

try
    mi_close; % ako postoji vec otvoren dokument, zatvori
catch
end

try % proveri da li je femm uopste otvoren
    main_minimize;
catch % ako nije otvori
    openfemm(1);    % minimizovan prozor = 1
end
newdocument(0); % tip dokumenta magnetics = 0

mi_hidegrid; % skloni tackice

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% uneti podaci o motoru

L = 70;             % duzina masine

% rotor info
RDInner = 43.4;     % precnik gvozdja rotora u delovima gde stoje magneti
RDOuter = 45.4;     % precnik gvozdja rotora u delovima gde su ispupcine
ShD = 21;           % precnik vratila
MPRel = 0.85;       % relativna ugaona irina magneta
MROuter = 24.5;     % udaljenost povrsine magneta od ose masine

% stator info
SDInner = 50.4;     % unutrasnji precnik gvozdja statora
SDSlots = 78.35;    % unutrasnji precnik zlebova statora
SDOuterS = 90;      % spoljasnji precnik gvozdja statora (za izvorno p=3)
STWRel = 3.845/20;  % relativna sirina statorskog zuba
STSRel = 1.8/20;    % relativno rastojanje izmedju dva susedna zuba
STC1Rel = 2.75/sqrt(20);  % rel. poluprecnik zaobljenja zleba kod jarma
STC2Rel = 0.7/sqrt(20);   % rel. poluprecnik zaobljenja zleba kod vrha zuba
STBThick = 1;       % debljina dela zuba koji pravi slovo T
STBDeg = 75;        % ugao izmedju tog dela i precnika

% izracunati podaci o motoru
% rotor
PPDeg = 180/p;          % ugaona irina pola
PPRad = pi/p;           % ugaona irina pola
RRInner = RDInner/2;    % poluprecnik gvozdja rotora kod magneta
RROuter = RDOuter/2;    % poluprecnik gvozdja rotora drugde
ShR = ShD/2;            % poluprecnik vratila
MPDeg = MPRel * PPDeg;  % ugaona irina magneta u stepenima
MPRad = MPDeg * pi/180; % ugaona irina magneta u radijanima
RTDeg = PPDeg - MPDeg;  % ugaona irina zuba izmedju magneta u stepenima
RTRad = RTDeg * pi/180; % ugaona irina zuba izmedju magneta u radijanima

% stator
SSDeg = 360/S;              % ugao koji obuhvata jedan zleb u stepenima
SSRad = SSDeg*pi/180;       % ugao koji obuhvata jedan zleb u radijanima
STWidth = STWRel*SSDeg;   % sirina statorskog zuba
STSpacing = STSRel*SSDeg; % rastojanje izmedju dva susedna zuba
STCorner1 = STC1Rel*sqrt(SSDeg);  % poluprecnik zaobljenja zleba kod jarma
STCorner2 = STC2Rel*sqrt(SSDeg);  % poluprecnik zaobljenja zleba kod vrha zuba
SDOuter = SDOuterS*3/p-SDSlots*(3/p-1); % spoljasnji precnik gvozdja statora
SRInner = SDInner/2;        % unutrasnji poluprecnik gvozdja statora
SRSlots = SDSlots/2;        % unutrasnji poluprecnik zlebova statora
SROuter = SDOuter/2;        % spoljasnji poluprecnik gvozdja statora

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% femm podeavanja
%maxseg0 = 0.5;     % ugaoni korak segmentacije lukova;
maxseg1 = 1;        % - gde je potrebna veca preciznost
maxseg2 = 5;        % segmentacija ce se raditi sa
maxseg3 = 10;       % manjim korakom
rotorGroup = 1;     % grupa elemenata koja cini rotor
statorGroup = 2;    % grupa elemenata koja cini stator
editGroup = 7;      % pomocna grupa elemenata za crtanje
delta = 0.1;        % rastojanje izmedju klizecih lukova u zazoru

if delta >= SRInner-MROuter
    error('delta too large');
end

mi_probdef(0,'millimeters','planar',1e-8,L);    % definicja problema
smartmesh(0);   % ogranicene sposobnosti kompjutera

SimNP = lcm(S,2*p)/S;       % broj polova neophodno da se simulira
SimNS = SimNP*S/(2*p);      % broj lebova u domenu simulacije
SimPDeg = PPDeg*SimNP;      % ugaona irina domena simulacije u stepenima
SimPRad = pi/180*SimPDeg;   % ugaona irina domena simulacije u radijanima
bc = mod(SimNP,2);          % odabir granicnog uslova na osnovu br polova

% materijali iz femmove biblioteke
mi_getmaterial('NdFeB 32 MGOe');
mi_getmaterial('1020 Steel');
mi_getmaterial('M-19 Steel');
mi_getmaterial('Air');

% boundary property koji oznacava kraj masine
mi_addboundprop('A=0',0,0,0,0,0,0,0,0,0,0,0);
% boundary property koji dozvoljava rotaciju rotorske grupe
mi_addboundprop('gapslide',0,0,0,0,0,0,0,0,6+bc,0,0);
% boundary property koji oznacava pocetak i kraj prostorne periode maine
mi_addboundprop('apbc1',0,0,0,0,0,0,0,0,4+bc,0,0);
mi_addboundprop('apbc2',0,0,0,0,0,0,0,0,4+bc,0,0);
mi_addboundprop('apbc3',0,0,0,0,0,0,0,0,4+bc,0,0);
mi_addboundprop('apbc4',0,0,0,0,0,0,0,0,4+bc,0,0);

es_iron = 1;    % velicina elemenata na apbc granici u gvodju
es_air = 0.5;   % velicina elemenata na apbc granici u zazoru

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% crtanje domena, zazora
%
% Crtanje se radi pomocu izmenjenih 'mi_z...' funkcija koje kao ulaz 
% prihvatuju koordinate u obliku kompleksnih brojeva umesto xy kao sto
% ocekuju funkcije ugradjene u OctaveFEMM paket. Izmenjene funkcije samo
% prosledjuju realan i imaginaran deo ugradjenim funkcijama. Koriste se
% radi jednostavnosti geometrijskih proracuna potrebnih za crtanje masine.
% Pomocne promenljive oznacene malim z su kompleksni brojevi sa jedinicnim
% modulom, dok velikim Z imaju neki drugi moduo. Realne pomocne promenljive
% ce biti oznacene velikim slovom R.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

z1 = exp(1j*SimPRad);
R1 = (MROuter+SRInner-delta)/2;
R2 = R1 + delta;

mi_zdrawarc(ShR,ShR*z1,SimPDeg,maxseg2);
mi_zdrawarc(SROuter,SROuter*z1,SimPDeg,maxseg2);
mi_zdrawline(ShR,R1);
mi_zdrawline(ShR*z1,R1*z1);
mi_zdrawline(R2,SROuter);
mi_zdrawline(R2*z1,SROuter*z1);

mi_zdrawarc(R1,R1*z1,SimPDeg,maxseg1);
mi_zdrawarc(R2,R2*z1,SimPDeg,maxseg1);

main_restore;
mi_zoomnatural;
main_minimize;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% crtanje rotora
%
% Crtanje periodicnih delova masine kao sto su zlebovi i magneti se radi
% tako sto se nacrta jedna perioda, selektuju sve linije i ubace u 'edit'
% grupu. Onda se ova grupa kopira pomocu 'mi_copyrotate2' funkcije.
% Geometrija, tacnije oblik magneta je pojednostavljen, ali minimalno, tako
% da ne utice na tacnost rezultata. (Unutrasnja strana magneta prati obim
% gvozdja rotora i sirina magneta je jednaka sirina zleba rotora.)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

z2 = exp(1j*RTRad/2);
z3 = z2*exp(1j*MPRad);

mi_zdrawarc(RROuter,RROuter*z2,RTDeg/2,maxseg2);
mi_zdrawline(RROuter*z2,RRInner*z2);
mi_zdrawarc(RRInner*z2,RRInner*z3,MPDeg,maxseg2);
mi_zdrawline(RROuter*z3,RRInner*z3);
mi_zdrawarc(RROuter*z3,RROuter*z2*z3,RTDeg/2,maxseg2);

mi_clearselected;
mi_zselectarcsegment(RROuter);
mi_zselectsegment(RROuter*z2);
mi_zselectarcsegment(RRInner*z2);
mi_zselectsegment(RROuter*z3);
mi_zselectarcsegment(RROuter*z2*z3);
mi_setgroup(editGroup);

mi_zdrawline(MROuter*z2,RRInner*z2);
mi_zdrawline(MROuter*z3,RRInner*z3);
mi_zdrawarc(MROuter*z2,MROuter*z3,MPDeg,maxseg1);

mi_clearselected;
mi_zselectsegment(MROuter*z2);
mi_zselectsegment(MROuter*z3);
mi_zselectarcsegment(MROuter*z3);
mi_setgroup(editGroup);

mi_clearselected;
mi_selectgroup(editGroup);
mi_copyrotate2(0,0,PPDeg,SimNP-1,4);
mi_selectgroup(editGroup);
mi_setgroup(rotorGroup);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% crtanje statora
%
% U slucaju statora je i blocklabel koji obelezava unutrasnjost zleba deo
% 'edit' grupe radi jednostavnosti (kod magneta ovo nije bilo moguce jer je
% potrebno da se smer magnetizacije menja naizmenicno). I ovde su uvedena
% pojednostavljenja koja, opet, ne uticu na tacnost rezultata. (Sirina zuba
% je konstantna, odnosno zub je sacinjen od dve paralelne linije.) Ipak,
% bilo je neophodno naci neke koordinater resavanjem jednacina (npr. Z12).
% Iskoriscena je simetrija zlebova i konjugacija kompleksnih brojeva kako
% bi se jednostavnije racunale koordinate (z16, Z18, Z19, Z20).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

z8 = exp(1j*SSRad/2);
z13 = z8*exp(-1j*pi/2);
a1 = angle(SRInner*z8+STSpacing/2*z13);
z9 = exp(1j*a1);
Z10 = SRInner*z9+STBThick*z8;
z11 = z8*exp(-1j*STBDeg*pi/180);
Z12 = Z10+(STWidth/2-imag(Z10))/imag(z11)*z11;

mi_zdrawarc(SRInner,SRInner*z9,a1*180/pi,maxseg1);
mi_zdrawline(SRInner*z9,Z10);
mi_zdrawline(Z10,Z12);

Z14 = SRSlots*z8;
Z15 = Z14 +(STWidth/2-imag(Z14))/imag(z13)*z13;

mi_zdrawline(Z12,Z15);
mi_zdrawline(Z14,Z15);

z16 = exp(1j*SSRad);
Z18 = conj(Z10)*z16;
Z19 = conj(Z12)*z16;
Z20 = conj(Z15)*z16;

mi_zdrawarc(SRInner*conj(z9)*z16,SRInner*z16,a1*180/pi,maxseg1);
mi_zdrawline(SRInner*conj(z9)*z16,Z18);
mi_zdrawline(Z18,Z19);

mi_zdrawline(Z19,Z20);
mi_zdrawline(Z20,Z14);

mi_zdrawline(Z10,Z18);

Z17 = (SRInner+SRSlots)/2*z8;

mi_zaddblocklabel(Z17)
mi_clearselected;
mi_zselectlabel(Z17);
mi_setblockprop('Air',1,0,0,0,editGroup,0);

mi_zcreateradius(Z15,STCorner1);
mi_zcreateradius(Z20,STCorner1);
mi_zcreateradius(Z12,STCorner2);
mi_zcreateradius(Z19,STCorner2);

mi_clearselected;
mi_zselectarcsegment(Z12);
mi_zselectarcsegment(0.9*Z15+0.1*Z20);
mi_zselectarcsegment(Z19);
mi_zselectarcsegment(0.9*Z20+0.1*Z15);
mi_setarcsegmentprop(maxseg3,0,0,editGroup);

mi_clearselected;
mi_zselectarcsegment(SRInner);
mi_zselectsegment(SRInner*z9);
mi_zselectsegment((Z10+Z12)/2);
mi_zselectsegment((Z12+Z15)/2);
mi_zselectsegment((Z15+Z14)/2);
mi_zselectsegment(abs(Z10)*z8);
mi_zselectarcsegment(SRInner*z16);
mi_zselectsegment(SRInner*conj(z9)*z16);
mi_zselectsegment((Z18+Z19)/2);
mi_zselectarcsegment(Z19);
mi_zselectsegment((Z19+Z20)/2);
mi_zselectsegment((Z20+Z14)/2);

mi_setgroup(editGroup);

mi_selectgroup(editGroup);
mi_copyrotate2(0,0,SSDeg,SimNS-1,4);

mi_selectgroup(editGroup);
mi_setgroup(statorGroup);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% podeavanja materijala i granicnih uslova
%
% Secifikacije materijala su uzete iz biblioteke integrisane u FEMM-u.
% Radijalni smer magnetizacije je definisan kao funkcija od theta, kao sto
% je opisano u uputstvu za FEMM.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%% blok labele (osim unutrasnjosti zlebova)

z4 = sqrt(z2*z3);
R3 = (RRInner+ShR)/2;

mi_zaddblocklabel(R3*z4);
mi_clearselected;
mi_zselectlabel(R3*z4);
mi_setblockprop('1020 Steel',1,0,0,0,rotorGroup,0);

% magnetizacija
Z5 = (RRInner+MROuter)/2*z4;
for i=1:SimNP
    mi_zaddblocklabel(Z5);
    mi_clearselected;
    mi_zselectlabel(Z5);
    mi_setblockprop('NdFeB 32 MGOe',1,0,0,...
        sprintf('theta+%i',mod(i+1,2)*180),rotorGroup,0);
    Z5 = Z5*exp(1j*PPRad);
end

Z6 = (R1+RROuter)/2*sqrt(z2);
Z7 = (R2+SRInner+STBThick)/2*z8;

mi_zaddblocklabel(Z6);
mi_clearselected;
mi_zselectlabel(Z6);
mi_setblockprop('Air',1,0,0,0,rotorGroup,0);

mi_zaddblocklabel(Z7);
mi_clearselected;
mi_zselectlabel(Z7);
mi_setblockprop('Air',1,0,0,0,statorGroup,0);

R4 = (SRSlots+SROuter)/2;
mi_zaddblocklabel(R4*z4);
mi_clearselected;
mi_zselectlabel(R4*z4);
mi_setblockprop('M-19 Steel',1,0,0,0,statorGroup,0);

%%%%%%%% boudary conditions

mi_clearselected;
mi_zselectarcsegment(ShR);
mi_setarcsegmentprop(maxseg2,'A=0',0,rotorGroup);
mi_clearselected;
mi_zselectarcsegment(SROuter);
mi_setarcsegmentprop(maxseg2,'A=0',0,statorGroup);

mi_clearselected;
mi_zselectsegment(ShR);
mi_zselectsegment(ShR*z1);
mi_setsegmentprop('apbc1',es_iron,0,0,rotorGroup);

mi_clearselected;
mi_zselectsegment(R1);
mi_zselectsegment(R1*z1);
mi_setsegmentprop('apbc2',es_air,0,0,rotorGroup);

mi_clearselected;
mi_zselectsegment(R2);
mi_zselectsegment(R2*z1);
mi_setsegmentprop('apbc3',es_air,0,0,statorGroup);

mi_clearselected;
mi_zselectsegment(SROuter);
mi_zselectsegment(SROuter*z1);
mi_setsegmentprop('apbc4',es_iron,0,0,statorGroup);

mi_clearselected;
mi_zselectarcsegment(R1);
mi_setarcsegmentprop(maxseg1,'gapslide',0,rotorGroup);
mi_clearselected;
mi_zselectarcsegment(R2);
mi_setarcsegmentprop(maxseg1,'gapslide',0,statorGroup);

mi_clearselected;
if nargin > 2
    fn = varargin{1};
    if strcmp(fn,'default')
        fn = sprintf('motor_%i_%i.fem',S,2*p);
    end
    mi_saveas(fn);
end
mi_minimize;
mi_restore;

end





































